import os.path
import logging

#
#  Check the system parameters
#  TODO: check for existence as well and the presence of the KRAKEN files
# 
def execute(system, runtime):
    
    checks = 0
    errors = 0
    
    # Convert all the paths to the Unix path
    print "Checking destination directory..."
    checks += 1
    system.addProperty("destinationDir", os.path.normpath(system.getProperty("destinationDir")))
    if (not os.path.isdir(system.getProperty("destinationDir"))):
        logging.critical("The destination directory does not exist:" + 
            system.getProperty("destinationDir"))
        errors += 1
            
            
    # the kraken dirs            
    print "Checking kraken directories..."
    checks += 1
    newDirs = []
    for a in system.getProperty("krakenDirs"):
        kdir = os.path.normpath(a)
        if (not os.path.isdir(kdir)):
           logging.critical("The kraken source directory does not exist:" + kdir)
           errors += 1
        newDirs.append(kdir)
    system.addProperty("krakenDirs", newDirs)
    
    # destination directory is a kraken dir.
    print "Checking if the destination dir is a kraken dir..."
    checks += 1
    if not (system.getProperty("destinationDir") in system.getProperty("krakenDirs")):
       logging.critical("The destination directory " + system.getProperty("destinationDir") + 
           " is not one of the kraken directories " + str(system.getProperty("krakenDirs")))
       errors += 1

    # the directory of the generated entries
    print "Checking the directory of generated entries..."
    checks += 1
    system.addProperty("generatedEntryDir", os.path.normpath(system.getProperty("generatedEntryDir")))
    if (not os.path.isdir(system.getProperty("generatedEntryDir"))):
        logging.critical("The directory of generated entries does not exist:" + 
            system.getProperty("generatedEntryDir"))
        errors += 1

    # the directory of the links
    print "Checking the directory of links..."
    checks += 1
    system.addProperty("linkDir", os.path.normpath(system.getProperty("linkDir")))
    if (not os.path.isdir(system.getProperty("linkDir"))):
        logging.critical("The directory of links does not exist:" + 
            system.getProperty("linkDir"))
        errors += 1
            
    # the staging dir            
    print "Checking the temporary (staging) directory..."
    system.addProperty("tempDir", os.path.normpath(system.getProperty("tempDir")))
    checks += 1
    if (not os.path.isdir(system.getProperty("tempDir"))):
        logging.critical("The temporary directory does not exist:" + 
            system.getProperty("tempDir"))
        errors += 1

    # the filter dir            
    print "Checking the filter directory..."
    checks += 1
    system.addProperty("filterDir", os.path.normpath(system.getProperty("filterDir")))
    if (not os.path.isdir(system.getProperty("filterDir"))):
        logging.critical("The filter directory does not exist:" + 
            system.getProperty("filterDir"))
        errors += 1
        
    if errors == 0:
        print "\n\nThere were no errors. " + str(checks) + " checks performed.\n"
    else:
        print "\n\n" + str(errors) + " found out of " + str(checks) + " checks performed.\n"
            
