import sys
import shutil
import os.path
import logging
import pickle
import kraken.tools.Utils as Utils
from kraken.entrytypes.MetaData import loadMetaData
import kraken.Constants
import kraken.presentation.Presentation as Presentation
import kraken.query.FilterEditor as FilterEditor
import kraken.data.web
import kraken.tools.TextUi as TextUi

def description(system, runtime):
    t  = "Generates a HTML file from the content of the kraken. The default\n"
    t += "location of this file is " + os.path.join(system.getProperty("destinationDir"), kraken.Constants.DEFAULT_HTML_FILE) + "\n" 
    t += "but it can be changed using the --outputfile option. \n\n"
    t += "It is usually used together with a query.\n"
    return t

#
#  Writes out a list of the cocoon in the html format.
#
def execute(system, runtime):
    metadata = loadMetaData(system, runtime)

    menuitems = [
      "Generate all saved queries",
      "Generate for new query"
    ]
    val = TextUi.inputMenu("What's your choice", menuitems, default = "Generate all saved queries")
    
    if val == "Generate for new query":
      que = FilterEditor.createQueryWithVerify(system, metadata)
      outputFile = os.path.join(system.getProperty("htmlDir"), 
                     kraken.Constants.DEFAULT_HTML_FILE)
      generateHtmlForQuery(system, metadata, que, outputFile) 
      return     
    if val == "Generate all saved queries":
      print "Generate all queries"
      lst = os.listdir(system.getProperty("filterDir"))
      filters = []
      links = []
      for a in lst: 
           if a[-4:] != kraken.Constants.EXTENSION_QUERY:
               continue
           filterFile = os.path.join(system.getProperty("filterDir"), a)
           FILE = open(filterFile)
           que = pickle.load(FILE)
           FILE.close()
           outputFile = os.path.join(system.getProperty("htmlDir"), 
                     a[:-4]+".html")
           generateHtmlForQuery(system, metadata, que, outputFile)
           links.append(a[:-4]+".html")
      print "Now generate index"
      indexFile = os.path.join(system.getProperty("htmlDir"), "index.html")
      temp = Presentation.generateIndexWithLinks(links)
      Utils.writeOutput(system, indexFile, temp)
      
       
def generateHtmlForQuery(system, metadata, query, outputFile):
      md = query.processMetaData(metadata)
      temp = Presentation.generateHtml(system, md)
      Utils.writeOutput(system, outputFile, temp)
      # and now copy the *.css
      scriptpath = kraken.data.web.__path__[0]
      cssfile = os.path.join(scriptpath, kraken.Constants.DEFAULT_CSS_FILE)
      if not os.path.isfile(cssfile):
          logging.warning("Could not found the css file: " + cssfile)
          return
      shutil.copy(cssfile, system.getProperty("htmlDir"))
      shutil.copy(
         os.path.join(scriptpath, "kraken_small.jpg"),
         system.getProperty("htmlDir")
       )
      print "HTML file generated successfully"
    
