import os.path
import urllib
from types import *
import time
import sys
import string
import logging
import strptime

from rdflib.URIRef import URIRef
from rdflib.Literal import Literal
from rdflib.BNode import BNode
from rdflib.Identifier import Identifier
from rdflib.Namespace import Namespace
from rdflib.constants import TYPE, ABOUT
from rdflib.TripleStore import TripleStore

import kraken.Constants
from TypicalNamespaces import *
import kraken.tools.TextUi as TextUi

class EntryBase:
    
    def __init__(self):        
        self.fields = {}
    
    # 
    #  Lists all the properties of the entry  
    #  MIGHT be rewritten by the inheriting classes.
    #
    def listPropertyNames(self):
        return self.fields.keys()
    
    # 
    #  Get a property of the entry.  
    #  MIGHT be rewritten by the inheriting classes.
    #
    def getProperty(self, propertyName):
        try:
            value =  self.fields[propertyName]
            return value            
        except KeyError:
            print "Property:" + propertyName + " does not exist in this entry:"
            print self
            return None
    
      
    # 
    #  Adds a new property to the entry. 
    #  MIGHT be rewritten by the inheriting classes.
    #
    def addProperty(self, property_name, property_value):
       self.fields[property_name] = property_value
        
    # 
    #  Returns the resource identifier of the entry 
    #  SHOULD be rewritten by the inheriting classes.
    #    
    def getResourceURI(self):
        return None
    
    # 
    #  Returns the mapping of the fieldname to RDF attribute 
    #  SHOULD be rewritten by the inheriting classes.
    #    
    def getMapping(self, field):
        return field
    
    '''
    Returns a hash of the namespaces (with the abbreviation as a key)
    might be rewritten or extended by the implementing classes.    
    '''
    def getNamespaces(self):
        return namespaces
            
    '''
    Returns the RDF class of the entry
    '''            
    def getRdfClass(self):
        return URIRef(NS_RDF["thing"])            
            
    # 
    #  Create store
    #
    def createStore(self):
       # create the store
       store = TripleStore()
       theClass = self.__class__
       # add the namespaces
       nspaces = self.getNamespaces()
       for ns in nspaces.keys():          
          store.prefix_mapping(ns, nspaces[ns])
       return store 
    
    #
    #    Writing the entry into an RDF file
    #        
    def writeEntryToRdf(self, fileName, store = None):
       if store == None:
          store = self.createStore()
       # add the attributes 
       entry = self.getResourceURI()
       for key in self.listPropertyNames():
          value = self.getProperty(key)
          store.add((entry, self.getMapping(key), Literal(value)))
       # make it conform to an owl class
       store.add((entry, URIRef(NS_RDF["type"]), self.getRdfClass()))
       # save and post-process the xml:base stuff which is not supported by RDFLib
       if fileName == None:
           return
       self.saveStoreAndPostProcess(store, fileName)
         
    def saveStoreAndPostProcess(self, store, fileName):
       store.save(fileName + ".tmp")
       FILE = open(fileName + ".tmp")
       TOCOPY = open(fileName,"w")
       change = 1
       while 1:
           line = FILE.readline()
           if line == "":
               break
           if change == 1 and string.find(line, "xmlns:base") != -1:
               line = string.replace(line, "xmlns:base", "xml:base")
               change = 0
           TOCOPY.write(line)
       FILE.close()
       TOCOPY.close()
       os.remove(fileName + ".tmp")
         
    #
    #    Reading the entry from an RDF file which contains only a 
    #    single entry.
    #        
    def readEntryFromRdf(self, fileName):       
       store = self.createStore()
       theClass = self.__class__          
       if (not os.path.exists(fileName)):
          logging.warning(fileName + "does not exist") 
          raise fileName + "does not exist"
       urlName = urllib.pathname2url(fileName)
       store.load(urlName)

       subject = None
       for a in store.subjects(predicate=URIRef(NS_RDF["type"])): 
          subject = a
          break
       if subject == None:
          print "Could not found a class - let's try with Any subject" 
          for a in store.subjects(): 
             subject = a
             break
          if subject == None:
              print "This is a mess"
              return
       resource = store[URIRef(subject)]
       # and now, fill in the entry from the loaded stuff
       for key in self.listPropertyNames():           
           rawvalue = resource[self.getMapping(key)]
           if rawvalue == None:
               print "Key "+key+ " could not be found, continuing"
               continue
           value = str(rawvalue)
           ### FIXME: This is wierd. What type is rawvalue?
           if value == "None":
               print "Key "+key+ " could not be found, continuing"
               continue
           #print key, value
           typeOfField = type(self.getProperty(key))
           if typeOfField in [TupleType,ListType,DictType,IntType,time.struct_time]:
               try:
                  self.addProperty(key, eval(value))
               except:
                  print sys.exc_info()
                  print "Unexpected value for key " + key + " = " + value
                  print "Can not eval. Keeping the default."                  
           else:
              self.addProperty(key, value)
           # check for errors in types, but accept a tuple instead of struct_time
           newtype = type(self.getProperty(key))
           if  (newtype != typeOfField and 
               not (typeOfField == time.struct_time and newtype == TupleType)):
               print "ERROR: The type of the field " + key + " changed after load!!!"
               print key, self.getProperty(key)
               
    '''
     Helper function: edit a field different datatypes.     
    '''
    def editField(self, fieldname, prompt = None):
      if prompt == None:
          prompt = fieldname
      val = self.getProperty(fieldname)
      typeOfField = type(val)
      if typeOfField == StringType:
         val2 = TextUi.inputWithDefault(prompt, val)
      elif typeOfField == ListType:
          val2 = TextUi.inputSequence(prompt, val)
      elif typeOfField == IntType:
          val2 = TextUi.inputInteger(prompt, val)
      elif (typeOfField == time.struct_time) or (typeOfField == TupleType and len(val)==9):  
          temp = TextUi.inputString(prompt, time.strftime("%a, %d %b %Y %H:%M:%S +0000", val))
          val2 = strptime.strptime(temp, "%a, %d %b %Y %H:%M:%S +0000")
      else:
          print fieldname + ": This datatype" + str(typeOfField) + " can not be currently edited in kraken"
          return
      if val != val2:
          self.addProperty(fieldname, val2)
           
      '''
        Prints out the fields. To be used in __str__ functions
      '''
    def strFields(self, properties = None, indent = "\t", ignoreNone = False):
        if properties == None:
            properties = self.listPropertyNames()
        tmp = ""
        for field in self.listPropertyNames():
            value = self.getProperty(field)
            if ignoreNone == True and (value == None or value == ""):
                #print "Ignoring"
                continue
            typeOfField = type(value)
            tmp += indent + field + " = " 
            # guess about a sequence of 9 integers that it is a time?
            if typeOfField == TupleType and len(value) == 9:   
                tmp += time.strftime("%a, %d %b %Y %H:%M:%S +0000", value)
            elif typeOfField == StringType:
                tmp += value
            elif typeOfField == ListType:
                tmp += str(value)
            elif typeOfField == IntType:
                tmp += str(value)
            else:
                tmp += "(" + str(typeOfField) + ")" + str(value)
            tmp += "\n"
        return tmp
        
    def __str__(self):
        return self.__class__.__name__ + ":\n" + self.strFields()
