import time
import os.path
import kraken.Constants
import pickle
from TableFormat import TableFormat
import FieldHelper
import kraken.entrytypes.MetaData as MetaData
              
        
# ***********************************************************
#  Generate web page
# ***********************************************************

def htmlBegin():
       temp = "";
       temp += "<html>\n"
       temp += "<head>\n"
       temp += '<link HREF="' + kraken.Constants.DEFAULT_CSS_FILENAME + '" REL="STYLESHEET" LANG="en.us">'
       temp += "</head>\n"
       return temp
       
def htmlEnd():
       temp = "</html>\n"
       return temp

def generatePageTable(maintable, smallTables):
    temp = '<table class=pagetable cols=2 cellspacing="20" width="100%">\n'
    # the header
    temp += '<tr>\n'
    temp += '<td class=pagetable-header colspan=2>\n'
    temp += '<table><tr><td><img src="kraken_small.jpg" height="80"/></td>'
    temp += '<td><h1>Generated by the Kraken</h1></td></tr></table>'
    temp += '</td>\n'
    temp += '</tr>\n'    
    
    # the main pages
    temp += '<tr>\n'
    temp += '<td class=pagetable-main width="70%">\n'
    temp += maintable
    temp += '</td>\n'    
    # here come the secondary tables
    temp += '<td class=pagetable-sidebar>\n'
    temp += '<table class=sidebar width="100%" cellspacing="0">\n'
    for smalltable in smallTables:
       temp += '<tr>\n'
       temp += '<td class=sidebar-cell>\n'
       temp += smalltable
       temp += '</td>\n'
       temp += '</tr>\n'
    temp += '</table>\n'
    
    temp += '</td>\n</tr>\n'    
    temp += '</table> <!-- pagetable-->\n'
    return temp
       
   
def generateMainTable(system, metadata):
   headertext = generateHeader(system, metadata)
   temp = ""
   tf = TableFormat()
   temp += tf.generateEntryTable(metadata, headertext)
   return temp 
   
       
def generateHtml(system, metadata):
   temp = htmlBegin()
   maintable = generateMainTable(system, metadata)
   smalltables = []
   smalltables.append(generateSmallTableWithQueries(system, metadata))
   smalltables.append(generateSmallTableFromQuery(system, "ResearchIdeas.qry", "Research Ideas"))
   smalltables.append(generateSmallTableFromQuery(system, "Conferences.qry", "Upcoming conferences"))
   temp += generatePageTable(maintable, smalltables)       
   temp += htmlEnd()
   return temp

def generateSmallTableFromQuery(system, queryName, header):
     metadata = MetaData.loadMetaData(system, None)
     filterFile = os.path.join(system.getProperty("filterDir"), queryName)
     FILE = open(filterFile)
     query = pickle.load(FILE)
     FILE.close()
     
     md = query.processMetaData(metadata)
     temp = '<table class=smalltable width="100%" cellspacing="0">'
     temp += '<tr>'
     temp += '<td class=smalltable-header width=100%>'
     temp += header
     temp += '</td>'
     temp += '</tr>'
     cellClass = "smalltable-even"
     for entry in md.entries:
         if cellClass == "smalltable-even":
             cellClass = "smalltable-odd"
         else:
             cellClass = "smalltable-even"
         temp += '<tr>\n<td class=' + cellClass + '>'
         temp += FieldHelper.linkLocalFiles(entry)
     temp +='</table>'
     return temp
    
       
def generateSmallTableWithQueries(system, metadata):       
      lst = os.listdir(system.getProperty("filterDir"))
      links = []
      for a in lst: 
           if a[-4:] != kraken.Constants.EXTENSION_QUERY:
               continue
           links.append(a[:-4]+".html")
      temp = '<table class=smalltable width="100%" cellspacing="0">'
      temp += '<tr>'
      temp += '<td class=smalltable-header width=100%>'
      temp += 'Saved queries'
      temp += '</td>'
      temp += '</tr>'
      cellClass = "smalltable-even"
      for a in links:
         if cellClass == "smalltable-even":
             cellClass = "smalltable-odd"
         else:
             cellClass = "smalltable-even"
         temp += '<tr>\n<td class=' + cellClass + '>'
         temp += '<a href="' + a + '">' + a[:-5] + '</a>'
      temp +='</table>'
      return temp
       
       
def generateIndexWithLinks(links):
       temp = htmlBegin()
       for a in links:
           temp += "<p><a href=" + a + ">" + a + "</a></p>" 
       temp += htmlEnd()
       return temp       
       
def generateHeader(system, metadata):
    temp = "<tr>\n<td class=maintable-header colspan=2>"
    temp +='<p><b>Generated on:</b>' + time.strftime("%a, %d %b %Y %H:%M:%S +0000", time.localtime())
    # fixme write out the query
    # temp +="<p><b>Filter all keywords:</b> " + runtime.selectKeywordsAll
    # temp +="<p><b>Filter any keywords:</b> " + runtime.selectKeywordsAny
    temp += "<pre>" + str(metadata.theQuery) + "</pre><br/>"
    temp += "<td></tr>"
    return temp


